package org.jboss.wikitransform.docbook;

import org.htmlparser.Node;
import org.htmlparser.Tag;
import org.htmlparser.tags.Bullet;
import org.htmlparser.tags.BulletList;
import org.htmlparser.tags.HeadTag;
import org.htmlparser.tags.LinkTag;
import org.htmlparser.tags.ParagraphTag;
import org.htmlparser.tags.TableColumn;
import org.htmlparser.tags.TableHeader;
import org.htmlparser.tags.TableRow;
import org.htmlparser.tags.TableTag;
import org.htmlparser.util.NodeList;

import java.io.PrintStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

/**
 * @author <a href="mailto:bill@burkecentral.com">Bill Burke</a>
 * @version $Revision: 1 $
 */
public class Chapter
{
   protected String name;
   protected URL url;
   protected String filename;

   protected List<Chapter> subchapters = new ArrayList<Chapter>();


   public Chapter(String name, URL url)
   {
      this.name = name;
      this.filename = name.replace(' ', '_').replace(',', '_').replace('@', '_').replace('/', '_').replace('\"', '_').replace('\'', '_');
      this.url = url;
   }

   public String getName()
   {
      return name;
   }

   public String getFileName()
   {
      return filename;
   }

   public URL getUrl()
   {
      return url;
   }

   public void setFilename(String filename)
   {
      this.filename = filename;
   }

   public void setUrl(URL url)
   {
      this.url = url;
   }

   public void setName(String name)
   {
      this.name = name;
   }

   public List<Chapter> getSubchapters()
   {
      return subchapters;
   }

   public void generate(PrintStream writer, int level)
   {
      String type = "chapter";
      if (level > 0) type = "sect1";
      String sect = "sect" + (level > 0 ? "2" : "1");
      writer.println("<" + type + " id=\"" + name + "\">");
      writer.println("<title>" + name + "</title>");
      if (url == null)
      {
         for (Chapter subchapter : subchapters)
         {
            subchapter.generate(writer, level + 1);
         }
         writer.println("</" + type + ">");
         return;
      }

      NodeList list = Util.getContent(url);
      if (list.elementAt(0) instanceof HeadTag) list.remove(0);

      toDocbook(writer, sect, list);

      for (Chapter subchapter : subchapters)
      {
         subchapter.generate(writer, level + 1);
      }

      writer.println("</" + type + ">");

   }

   private void toDocbook(PrintStream writer, String sect, NodeList list)
   {
      boolean p = false;
      boolean section = false;
      for (Node node : list.toNodeArray())
      {
         if (node instanceof ParagraphTag)
         {
            if (p) writer.println("</para>");
            writer.println("<para>");
            p = true;
         }
         else if (node instanceof HeadTag)
         {
            if (p)
            {
               p = false;
               writer.println("</para>");
            }
            if (section) writer.println("</" + sect + ">");
            writer.println("<" + sect + ">");
            writer.println("<title>" + node.toPlainTextString() + "</title>");
            section = true;
         }
         else if (node instanceof LinkTag)
         {
            writer.print(node.getChildren().toHtml());
         }
         else if (node instanceof TableTag)
         {
            TableTag table = (TableTag) node;
            writer.println("<table frame=\"topbot\">");
            int maxColumns = 0;
            for (TableRow row : table.getRows())
            {
               if (row.getColumnCount() > maxColumns) maxColumns = row.getColumnCount();
            }
            writer.println("<tgroup cols=\"" + maxColumns + "\" rowsep=\"1\" colsep=\"1\">");
            if (table.getRows()[0].hasHeader())
            {
               TableRow row = table.getRows()[0];
               writer.println("<thead>");
               writer.println("<row>");
               for (TableHeader header : row.getHeaders())
               {
                  writer.println("<entry>");
                  toDocbook(writer, sect, header.getChildren());
                  writer.println("</entry>");
               }
               writer.println("</row>");
               writer.println("</thead>");

            }
            writer.println("<tbody>");
            for (TableRow row : table.getRows())
            {
               if (row.hasHeader()) continue;
               writer.println("<row>");
               for (TableColumn column : row.getColumns())
               {
                  writer.println("<entry>");
                  toDocbook(writer, sect, column.getChildren());
                  writer.println("</entry>");
               }
               writer.println("</row>");

            }
            writer.println("</tbody>");
            writer.println("</tgroup>");
            writer.println("</table>");
         }
         else if (node instanceof BulletList)
         {
            BulletList bulletList = (BulletList) node;
            String tag = (bulletList.getTagName().equalsIgnoreCase("ul")) ? "itemizedlist" : "orderedlist";

            writer.println("<" + tag + ">");
            toDocbook(writer, sect, bulletList.getChildren());
            writer.println("</" + tag + ">");
         }
         else if (node instanceof Bullet)
         {
            writer.println("<listitem>");
            toDocbook(writer, sect, ((Bullet) node).getChildren());
            writer.println("</listitem>");
         }
         else if (node instanceof Tag)
         {
            Tag tag = (Tag) node;
            if (tag.getTagName().equalsIgnoreCase("pre"))
            {

               if (tag.isEndTag()) writer.print("</programlisting>");
               else
               {
                  writer.print("<programlisting>");
                  if (node.getChildren() != null) writer.println(node.getChildren().toHtml());
               }
            }
         }
         else
         {
            writer.print(node.toHtml());
         }
      }
      if (p) writer.println("</para>");
      if (section) writer.println("</" + sect + ">");
   }

}
